#pragma once

/* ColorCalibration.h
*
*  MIT License
*
*  Copyright (c) 2020-2023 awawa-dev
*
*  Project homesite: https://github.com/awawa-dev/HyperHDR
*
*  Permission is hereby granted, free of charge, to any person obtaining a copy
*  of this software and associated documentation files (the "Software"), to deal
*  in the Software without restriction, including without limitation the rights
*  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
*  copies of the Software, and to permit persons to whom the Software is
*  furnished to do so, subject to the following conditions:
*
*  The above copyright notice and this permission notice shall be included in all
*  copies or substantial portions of the Software.

*  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
*  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
*  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
*  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
*  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
*  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
*  SOFTWARE.
*/

#ifndef PCH_ENABLED
	#include <QString>
#endif

#include <utils/ChannelCalibration.h>
#include <utils/ColorSpaceCalibration.h>

class ColorCalibration
{
private:
	QString _id;

	std::unique_ptr<ChannelCalibration> _blackCalibration;
	std::unique_ptr<ChannelCalibration> _whiteCalibration;
	std::unique_ptr<ChannelCalibration> _redCalibration;
	std::unique_ptr<ChannelCalibration> _greenCalibration;
	std::unique_ptr<ChannelCalibration> _blueCalibration;
	std::unique_ptr<ChannelCalibration> _cyanCalibration;
	std::unique_ptr<ChannelCalibration> _magentaCalibration;
	std::unique_ptr<ChannelCalibration> _yellowCalibration;

	std::unique_ptr<ColorSpaceCalibration> _colorspaceCalibration;
public:
	ColorCalibration(uint8_t instance, const QJsonObject& adjustmentConfig);

	QString getId();
	void setBackLightEnabled(bool enabled);
	void calibrate(ColorRgb& color);
	void putCurrentConfig(QJsonObject& adjustment);
	void updateConfig(const QJsonObject& adjustment);
};
